import os
from os import name as os_name
import sys
from bz2file import BZ2File
from pdb import set_trace as bp
from pprint import pprint
from __init__ import Parser

# add parent dir to PATH
sys.path.insert(1, os.path.join(sys.path[0], '..'))

if os_name == 'nt':
    bzfile_path = 'C:/Users/Gregor/Downloads/dewiktionary-latest-pages-articles-multistream.xml.bz2'
else:
    bzfile_path = '/Users/gregor/Downloads/dewiktionary-latest-pages-articles-multistream.xml.bz2'
bz = BZ2File(bzfile_path)
collection = set()
for record in Parser(bz):

    # German entries missig IPA:
    # if record['title'] not in collection and \
    #     'langCode' in record and record['langCode'] == 'de' and \
    #     'pos' in record and 'Abkürzung' not in record['pos'] and \
    #     'ipa' not in record:
    #     print(record['title'])

    # if 'langCode' in record:
    #     print(record['langCode'])

    if record['title'] == 'Drittes Reich':
        pprint(record)
    # if 'syllables' in record:
    #     print(record['title'])
    #     print(record['syllables'])
    #     print()

    # # German entries missig syls:
    # if record['title'] not in collection and \
    #     'langCode' in record and record['langCode'] == 'de' and \
    #     'pos' in record and 'Abkürzung' not in record['pos'] and \
    #     'syllables' not in record:
    #     print(record['title'])

    collection.add(record['title'])



import re
from pprint import pprint
# from pdb import set_trace as bp

"""
reference:
https://de.wiktionary.org/wiki/Kategorie:Wiktionary:Flexionstabelle_(Deutsch)
https://de.wiktionary.org/wiki/Hilfe:Flexionstabellen
"""


wanted_table_names = [
    'Deutsch Adjektiv Übersicht',
    'Deutsch Adverb Übersicht',
    'Deutsch Eigenname Übersicht',
    'Deutsch Nachname Übersicht',
    'Deutsch Pronomen Übersicht',
    'Deutsch Substantiv Übersicht',
    'Deutsch Substantiv Übersicht - sch',
    'Deutsch adjektivisch Übersicht',
    'Deutsch Toponym Übersicht',
    'Deutsch Verb Übersicht',
]


def find_table(text):
    re_string = '({{(' + '|'.join(wanted_table_names) + ')[^}]+}})'
    match_table = re.search(re_string, text)
    if not match_table:
        return False
    return match_table.group(1)


def clean_text(text):
    # strip comments, <ref>-tags etc.
    text = re.sub(r'<[^>]+>', ' ', text)

    # trim
    text = text.strip()

    return text


def find_table_values(table_string):
    table_tuples = re.findall(r'(?:^\|([^=]+)=([^\n]+)$)+', table_string, re.MULTILINE)
    if not table_tuples:
        return False

    # normalize values
    result = {}
    for (key, value) in table_tuples:
        if key.startswith('Bild'):
            continue
        if key in ('Flexion', 'Weitere Konjugationen'):
            continue

        # clean text
        text = clean_text(value)

        # genus
        if key in ['Genus', 'Genus 1', 'Genus 2', 'Genus 3', 'Genus 4']:
            # the Genus of plural words is set to 0 (or other value)
            # reference: https://de.wiktionary.org/wiki/Wiktionary:Teestube/Archiv/2015/11#Genus_in_der_Flexionstabelle_bei_Pluralw%C3%B6rtern
            # -> normalize
            text = text.lower()
            if text not in ['f', 'm', 'n']:
                text = None

        # none dash to None type
        if text in ('—', '-', '–'):
            text = None

        result[key] = text

    return result if result.keys() else False


def init(title, text, current_record):
    table_string = find_table(text)
    if not table_string:
        return False

    table_dict = find_table_values(table_string)
    if table_dict is False:
        return False

    return {'flexion': table_dict}



import re
from pdb import set_trace as bp

"""
Reference: https://de.wiktionary.org/wiki/Hilfe:Aussprache

The WiktionaryDe community decided in 2015 to only add IPA transcriptions
for the lemma (and not for inflected forms; these are on a seperate page).
Unfortunately there are still many pages with inflected forms.
For example: "Schweizerdeutsch" and Singular 2 "(das) Schweizerdeutsche":
    - :{{IPA}} {{Lautschrift|ˈʃvaɪ̯t͡sɐˌdɔɪ̯t͡ʃ}}, {{Sg.2}} {{Lautschrift|ˈʃvaɪ̯t͡sɐˌdɔɪ̯t͡ʃə}}

Sometimes they are just seperated by comma and no additional info about the grammatical form is present:
    Example "Aserbaidschanisch" and Singular 2 "(das) Aserbaidschanische":
    - :{{IPA}} {{Lautschrift|ˌazɐbaɪ̯ˈd͡ʒaːnɪʃ}}, {{Lautschrift|ˌazɐbaɪ̯ˈd͡ʒaːnɪʃə}}

In the last example we can't say whether the two entries are just two different IPA
transcriptions for the same word or for two different words (lemma & inflected form). 

Temporary solution:
Keep the first entry. Check if the last IPA letter of every other entry is the same
as the one in the first match (because it's likely that the last character doesn't
change in different IPA transcriptions for the same word)

"""

vowels = r'aɪ̯|aʊ̯|ɛɪ̯|ɔɪ̯|ʊɪ̯|aː|eː|ɛː|iː|oː|øː|õː|uː|yː|ɨ|i̯|ʉ|ɯ|ɪ|ʏ|ʊ|ø|ɑ|ɘ|ɵ|ɤ|ə|ɛ|œ|ɜ|ɞ|ʌ|ɔ|æ|ɐ|ɶ|ɒ|ã|ɐ̯|a|e|i|o|u|y'
consonants = r'ʈ|ɖ|ɟ|ɢ|ʔ|ɸ|β|v|θ|ð|ʃ|ʒ|ʂ|ʐ|ç|ʝ|l̩|ɣ|χ|ʁ|ħ|ʕ|ɦ|ɬ|ɮ|ɱ|m̩|ɱ̍|ɱ̩|n̩|ɳ|ɲ|ŋ|ŋ̍|ŋ̩|ɴ|ʙ|ʀ|ⱱ|ɾ|ʦ|ʧ|ʤ|ɽ|ɺ|ʋ|ɹ|ɻ|ɰ|ɭ|ʎ|ʟ|p|b|t|d|c|k|ɡ|q|f|s|z|x|h|m|n|r|l|j|ɫ'
ipa_letters_re = re.compile(r'(' + vowels + '|' + consonants + ')')


def init(title, text, current_record):
    # search line by line
    # headline {{Aussprache}} must come first
    # break at empty newline
    lines = text.split('\n')
    found_head = False
    found_ipa = []
    for line in lines:
        if line.startswith('{{Aussprache}}'):
            found_head = True
            continue
        if found_head is False:
            continue
        # break on empty newline
        if line.strip() == '':
            break
        # find IPA string(s)
        match_ipa = re.findall(r'{{Lautschrift\|([^}]+)}}', line)
        if not match_ipa:
            continue
        found_ipa = [x.strip() for x in match_ipa if x != '…' and x.strip() != '']
        if found_ipa:
           break
    
    if not found_ipa:
        return False

    # workaround for problem described above
    # keep IPA strings, that have the same ending as the first one in the row
    wanted_last_ipa_char = None
    result = [found_ipa[0]]
    for idx, ipa_str in enumerate(found_ipa):
        try:
            last_ipa_char = ipa_letters_re.findall(ipa_str)[-1]
        except IndexError:
            if idx == 0:
                return False
            else:
                break
        if idx == 0:
            wanted_last_ipa_char = last_ipa_char
            continue
        if last_ipa_char == wanted_last_ipa_char and ipa_str not in result:
            result.append(ipa_str)
        else:
            break

    return {'ipa': result}



from setuptools import setup, find_packages

with open('README.md', 'r', encoding='utf-8') as fh:
    readme = fh.read()

setup(name='wiktionary_de_parser',
    version='0.8.1',
    author='Gregor Weichbrodt',
    author_email='gregorweichbrodt@gmail.com',
    description='Extracts data from German Wiktionary dump files. Allows you to add your own extraction methods 🚀',
    long_description=readme,
    long_description_content_type='text/markdown',
    include_package_data= True,
    classifiers=[
        'Programming Language :: Python :: 3.7',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Natural Language :: German',
        'Topic :: Text Processing :: Markup :: XML'
    ],
    url='https://github.com/gambolputty/wiktionary_de_parser',
    keywords='wiktionary xml parser data-extraction german nlp',
    license='MIT',
    packages=find_packages(),
    python_requires='>=3.7',
    install_requires=['lxml', 'pyphen'],
    project_urls={
        'Bug Reports': 'https://github.com/gambolputty/wiktionary_de_parser/issues',
        'Source': 'https://github.com/gambolputty/wiktionary_de_parser',
    })



import os
import re
import sys
from os.path import dirname, abspath

parent_path = dirname(dirname(abspath(__file__)))

# https://de.wiktionary.org/wiki/Hilfe:Sprachcodes
langcodes = {}
with open(f'{parent_path}/assets/sprachcodes_iso639-1.txt', encoding='utf-8') as f:
    lines = f.read().split('\n')
    for line in lines:
        x = line.split(',')
        langcodes[x[0]] = x[1]


def init(title, text, current_record):
    match_lang_name = re.search(r'=== ?{{Wortart(?:-Test)?\|[^}|]+\|([^}|]+)(?:\|[^}|]+)*}}', text)
    if not match_lang_name:
        return False

    lang_name = match_lang_name.group(1) if match_lang_name.group(1) else match_lang_name.group(2)
    lang_name = lang_name.strip()

    # get lang code
    result = { 'lang': lang_name }

    lang_name_low = lang_name.lower()
    if lang_name_low in langcodes:
        result['langCode'] = langcodes[lang_name_low]

    return result
